"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.default = void 0;

var _helperPluginUtils = require("@babel/helper-plugin-utils");

var _types = require("@babel/types");

function getTagNameByJSXElement(path) {
  const node = path.node || path;
  const openingElement = node.openingElement;
  const tagNameIdentifier = openingElement.name;
  return tagNameIdentifier.name;
}

function isCustomJsxTag(path) {
  const tagName = getTagNameByJSXElement(path);
  return !tagName || tagName[0] === tagName[0].toUpperCase();
}

function toHump(name) {
  return name.replace(/\-(\w)/g, function (all, letter) {
    return letter.toUpperCase();
  });
}

var _default = (0, _helperPluginUtils.declare)((api, options) => {
  api.assertVersion(7);
  const prefix = options.prefix,
        importee = options.importee;
  return {
    name: "jsx-rename",

    pre(state) {
      this.renameComponentCache = new Set();
    },

    post(state) {
      this.renameComponentCache = null;
    },

    visitor: {
      Program: {
        exit(path) {
          const imports = [...this.renameComponentCache];

          if (imports && imports.length > 0) {
            imports.forEach(it => {
              const tpl = api.template(`import ${it} from "${importee}/lib/components/${it}";`, {
                sourceType: "module"
              });
              path.node.body.unshift(tpl());
            }); // const marsImport = api.template(`import {${imports.join(',')}} from "${importee}";`, { sourceType: "module" });
            // console.log(`import {${imports.join(',')}} from "${importee}";`, marsImport)
          }
        }

      },

      JSXElement(path) {
        if (isCustomJsxTag(path)) {
          // 若元素为自定义组件
          // 不予处理
          return;
        }

        const openingElement = path.get('openingElement');
        const closingElement = path.get('closingElement');
        [openingElement, closingElement].forEach(el => {
          if (!el.node) return;
          const namePath = el.get('name');
          const tagName = namePath.node.name;
          let transferTagName = toHump(tagName);
          transferTagName = transferTagName.replace(/^\S/, s => s.toUpperCase());
          namePath.replaceWith((0, _types.jsxIdentifier)(transferTagName));
          this.renameComponentCache.add(transferTagName);
        });
      }

    }
  };
});

exports.default = _default;